from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcx_camf_d1_magnitude_on_richter_scale = QzssDcrDefinition(
    {
        0: "1.0-1.9 - Micro",
        1: "2.0-2.9 - Minor",
        2: "3.0-3.9 - Minor",
        3: "4.0-4.9 - Light",
        4: "5.0-5.9 - Moderate",
        5: "6.0-6.9 - Strong",
        6: "7.0-7.9 - Major",
        7: "8.0-8.9 - Great",
        8: "9.0 and greater - Great",
    },
    undefined='Undefined magnitude on richter scale (Code: %d)'
)

qzss_dcx_camf_d2_seismic_coefficient = QzssDcrDefinition(
    {
        0: '2',
        1: '3',
        2: '4',
        3: '5 weak',
        4: '5 strong',
        5: '6 weak',
        6: '6 strong',
        7: '7',
    },
    undefined='Undefined seismic coefficient (Code: %d)'
)

qzss_dcx_camf_d3_azimuth_from_centre_of_main_ellipse_to_epicentre = {
    0: 0.0,
    1: 22.5,
    2: 45.0,
    3: 67.5,
    4: 90.0,
    5: 112.5,
    6: 135.0,
    7: 157.5,
    8: 180.0,
    9: 202.5,
    10: 225.0,
    11: 247.5,
    12: 270.0,
    13: 292.5,
    14: 315.0,
    15: 337.5,
}

qzss_dcx_camf_d4_vector_length_between_centre_of_main_ellipse_and_epicentre = {
    0: 0.25,
    1: 0.5,
    2: 0.75,
    3: 1,
    4: 2,
    5: 3,
    6: 5,
    7: 10,
    8: 20,
    9: 30,
    10: 40,
    11: 50,
    12: 70,
    13: 100,
    14: 150,
    15: 200,
}

qzss_dcx_camf_d5_wave_height = QzssDcrDefinition(
    {
        0: 'H ≤ 0.5m',
        1: '0.5m < H ≤ 1.0m',
        2: '1.0m < H ≤ 1.5m',
        3: '1.5m < H ≤ 2.0m',
        4: '2.0m < H ≤ 3.0m',
        5: '3.0m < H ≤ 5.0m',
        6: '5.0m < H ≤ 10.0m',
        7: 'H > 10.0m',
    },
    undefined='Undefined wave height (Code: %d)'
)

qzss_dcx_camf_d6_temperature_range = QzssDcrDefinition(
    {
        0: 'T ≤ -30°C',
        1: '-30°C < T ≤ -25°C',
        2: '-25°C < T ≤ -20°C',
        3: '-20°C < T ≤ -15°C',
        4: '-15°C < T ≤ -10°C',
        5: '-10°C < T ≤ -5°C',
        6: '-5°C < T ≤ 0°C',
        7: '0°C < T ≤ 5°C',
        8: '5°C < T ≤ 10°C',
        9: '10°C < T ≤ 15°C',
        10: '15°C < T ≤ 20°C',
        11: '20°C < T ≤ 25°C',
        12: '25°C < T ≤ 30°C',
        13: '30°C < T ≤ 35°C',
        14: '35°C < T ≤ 45°C',
        15: 'T > 45°C',
    },
    undefined='Undefined temperature (Code: %d)'
)

qzss_dcx_camf_d7_hurricane_category = QzssDcrDefinition(
    {
        0: 'Category 1/5 hurricane: Very dangerous winds will produce some damage. Winds range from 120 to 150 km/h. Falling debris could strike people, livestock and pets, and older mobile homes could be destroyed. Protected glass windows will generally make it through the hurricane without major damage. Frame homes, apartments and shopping centres may experience some damage, and snapped power lines could result in short-term power outages.',
        1: 'Category 2/5 hurricane: Extremely dangerous winds will cause extensive damage. Winds range between 150 and 180 km/h. There is a bigger risk of injury or death to people, livestock and pets from flying debris. Older mobile homes will likely be destroyed, and debris can ruin newer mobile homes, too. Frame homes, apartment buildings and shopping centres may see major roof and siding damage, and many trees will be uprooted. Residents should expect near total power loss after a Category 2 hurricane, with outages lasting anywhere from a few days to a few weeks.',
        2: 'Category 3/5 hurricane: Devastating damage will occur. In a Category 3 hurricane, winds range from 180 to 210 km/h. There is a high risk of injury or death to people, livestock and pets from flying and falling debris. Nearly all older mobile homes will be destroyed, and most new ones will experience significant damage. Even well-built frame homes, apartments and industrial buildings will likely experience major damage, and the storm will uproot many trees that may block roads. Electricity and water will likely be unavailable for several days to a few weeks after the storm.',
        3: 'Category 4/5 hurricane: Catastrophic damage will occur. During a Category 4 hurricane, winds range from 210 to 250 km/h. At these speeds, falling and flying debris poses a very high risk of injury or death to people, pets and livestock. Again, most mobile homes will be destroyed, even newer ones. Some frame homes may totally collapse, while well-built homes will likely see severe damage to their roofs, and apartment buildings can experience damage to upper floors. A Category 4 hurricane will blow out most windows on high-rise buildings, uproot most trees and will likely down many power lines.',
        4: 'Category 5/5 hurricane: Catastrophic damage will occur. In a Category 5 hurricane, the highest category hurricane, winds are 250 km/h or higher. People, livestock and pets can be in danger from flying debris, even indoors. Most mobile homes will be completely destroyed, and a high percentage of frame homes will be destroyed. Commercial buildings with wood roofs will experience severe damage, metal buildings may collapse and high-rise windows will nearly all be blown out. A Category 5 hurricane is likely to uproot most trees and ruin most power poles. People should expect long-term water shortages and power outages.',
    },
    undefined='Undefined hurricane category (Code: %d)'
)

qzss_dcx_camf_d8_wind_speed = QzssDcrDefinition(
    {
        0: 'Beaufort 0 - 0km/h < v < 1km/h - Calm',
        1: 'Beaufort 1 - 1km/h < v < 5km/h - Light Air',
        2: 'Beaufort 2 - 6km/h < v < 11km/h - Light Breeze',
        3: 'Beaufort 3 - 12km/h < v < 19km/h - Gentle Breeze',
        4: 'Beaufort 4 - 20km/h < v < 30km/h - Moderate Breeze',
        5: 'Beaufort 5 - 31km/h < v < 39km/h - Fresh Breeze',
        6: 'Beaufort 6 - 40km/h < v < 50km/h - Strong Breeze',
        7: 'Beaufort 7 - 51km/h < v < 61km/h - Near Gale',
        8: 'Beaufort 8 - 62km/h < v < 74km/h - Gale',
        9: 'Beaufort 9 - 75km/h < v < 88km/h - Strong Gale',
        10: 'Beaufort 10 - 89km/h < v < 102km/h - Storm',
        11: 'Beaufort 11 - 103km/h < v < 117km/h - Violent Storm',
        12: 'Beaufort 12 - v > 118km/h - Hurricane',
    },
    undefined='Undefined wind speed (Code: %d)'
)

qzss_dcx_camf_d9_rainfall_amounts = QzssDcrDefinition(
    {
        0: 'p ≤ 2.5mm/h',
        1: '2.5mm/h < p ≤ 7.5mm/h',
        2: '7.5mm/h < p ≤ 10mm/h',
        3: '10mm/h < p ≤ 20mm/h',
        4: '20mm/h < p ≤ 30mm/h',
        5: '30mm/h < p ≤ 50mm/h',
        6: '50mm/h < p ≤ 80mm/h',
        7: '80mm/h < p',
    },
    undefined='Undefined rainfall amounts (Code: %d)'
)

qzss_dcx_camf_d10_damage_category = QzssDcrDefinition(
    {
        0: 'Category 1 - Very dangerous winds will produce some damage. Scale 1 and Intensity 1',
        1: 'Category 2 - Extremely dangerous winds will cause extensive damage. Scale 1 and Intensity 2',
        2: 'Category 3 - Devastating damage will occur. Scale 1 and Intensity 3',
        3: 'Category 4 - Catastrophic damage will occur. Scale 2 and Intensity 1',
        4: 'Category 5 - Catastrophic damage will occur. Scale 2 and Intensity 2',
        5: 'Category 5 - Catastrophic damage will occur. Scale 3 and Intensity 3',
    },
    undefined='Undefined damage category (Code: %d)'
)

qzss_dcx_camf_d11_tornado_probability = QzssDcrDefinition(
    {
        0: 'Non-Threatening - Threat: No discernible threat to life and property. Minimum Action: Listen for forecast changes; review tornado safety rules. Potential Impact: None expected; strong wind gusts may still occur.',
        1: 'Very Low - Threat: A very low threat to life and property. Minimum Action: Preparations should be made for a very low likelihood (or a 2 to 4% probability) of tornadoes; isolated tornadoes of F0 to F1 intensity possible. Potential Impact: The potential for isolated locations to experience minor to moderate tornado damage.',
        2: 'Low - Threat: A low threat to life and property. Minimum Action: Preparations should be made for a low likelihood (or a 5 to 14% probability) of tornadoes; scattered tornadoes of F0 to F1 intensity possible. Potential Impact: The potential for scattered locations to experience minor to moderate tornado damage.',
        3: 'Moderate - Threat: A moderate threat to life and property. Minimum Action: Preparations should be made for a moderate likelihood (or a 15 to 29% probability) of tornadoes; many tornadoes (even families) of F0 to F1 intensity possible. Potential Impact: The potential for many locations to experience minor to moderate tornado damage (see below). Some tornadoes may have longer damage tracks.',
        4: 'High - Threat: A high threat to life and property. Minimum Action: Preparations should be made for a high likelihood (or a 30 to 44% probability) of tornadoes; scattered tornadoes possible with isolated tornadoes of F2 to F5 intensity also possible. Potential Impact: The potential for isolated locations to experience major tornado damage (see below), among scattered locations of minor to moderate tornado damage. Some tornadoes may have longer damage tracks.',
        5: 'Extreme - Threat: An extreme threat to life and property. Minimum Action: Preparations should be made for a very high likelihood (or a 45% probability or greater) of tornadoes; many tornadoes (even families) possible with scattered tornadoes of F2 to F5 intensity also possible. Potential Impact: The potential for scattered locations to experience major tornado damage (see below), among many locations of minor to moderate tornado damage. Some tornadoes may have longer damage tracks',
    },
    undefined='Undefined tornado probability (Code: %d)'
)

qzss_dcx_camf_d12_hail_scale = QzssDcrDefinition(
    {
        0: 'H0 Hard hail. Typical hail diameter of 5 mm, No damage',
        1: 'H1 Potentially damaging. Typical hail diameter of 5-15 mm. Slight general damage to plants, crops',
        2: 'H2 Significant. Typical hail diameter of 10-20 mm. Slight general damage to fruit, crops, vegetation',
        3: 'H3 Severe. Typical hail diameter of 20-30 mm (size of a walnut). Severe damage to fruit and crops, damage to glass and plastic structures, paint and wood scored',
        4: 'H4 Severe. Typical hail diameter of 25-40 mm (size of a squash ball). Widespread glass damage, vehicle bodywork damage',
        5: 'H5 Destructive. Typical hail diameter of 30-50 mm (size of a golf ball). Wholesale destruction of glass, damage to tiled roofs, significant risk of injuries',
        6: 'H6 Destructive. Typical hail diameter of 40-60 mm. Bodywork of grounded aircraft dented, brick walls pitted',
        7: 'H7 Destructive. Typical hail diameter of 50-75 mm (size of a tennis ball). Severe roof damage, risk of serious injuries',
        8: 'H8 Destructive. Typical hail diameter of 60-90 mm (size of a large orange). Severe damage to aircraft bodywork',
        9: 'H9 Super Hailstorms. Typical hail diameter of 75-100 mm (size of a grapefruit). Extensive structural damage. Risk of severe or even fatal injuries to persons caught in the open',
        10: 'H10 Super Hailstorms. Typical hail diameter > 100 mm (size of a melon). Extensive structural damage. Risk of severe or even fatal injuries to persons caught in the open',
    },
    undefined='Undefined tornado probability (Code: %d)'
)

qzss_dcx_camf_d13_visibility = QzssDcrDefinition(
    {
        0: 'Dense fog: visibility < 20m',
        1: 'Thick fog: 20m < visibility < 200m',
        2: 'Moderate fog: 200m < visibility < 500m',
        3: 'Light fog: 500m < visibility < 1000m',
        4: 'Thin fog: 1km < visibility < 2km',
        5: 'Haze: 2km < visibility < 4km',
        6: 'Light haze: 4km < visibility < 10km',
        7: 'Clear: 10km < visibility < 20km',
        8: 'Very clear: 20km < visibility < 50km',
        9: 'Exceptionally clear: visibility > 50km',
    },
    undefined='Undefined visibility (Code: %d)'
)

qzss_dcx_camf_d14_snow_depth = QzssDcrDefinition(
    {
        0: '0cm < daily snow depth ≤ 20cm',
        1: '20cm < daily snow depth ≤ 40cm',
        2: '40cm < daily snow depth ≤ 60cm',
        3: '60cm < daily snow depth ≤ 80cm',
        4: '80cm < daily snow depth ≤ 100cm',
        5: '100cm < daily snow depth ≤ 120cm',
        6: '120cm < daily snow depth ≤ 140cm',
        7: '140cm < daily snow depth ≤ 160cm',
        8: '160cm < daily snow depth ≤ 180cm',
        9: '180cm < daily snow depth ≤ 200cm',
        10: '200cm < daily snow depth ≤ 220cm',
        11: '220cm < daily snow depth ≤ 240cm',
        12: '240cm < daily snow depth ≤ 260cm',
        13: '260cm < daily snow depth ≤ 280cm',
        14: '280cm < daily snow depth ≤ 300cm',
        15: '300cm < daily snow depth ≤ 320cm',
        16: '320cm < daily snow depth ≤ 340cm',
        17: '340cm < daily snow depth ≤ 360cm',
        18: '360cm < daily snow depth ≤ 380cm',
        19: '380cm < daily snow depth ≤ 400cm',
        20: '400cm < daily snow depth ≤ 420cm',
        21: '420cm < daily snow depth ≤ 440cm',
        22: '440cm < daily snow depth ≤ 460cm',
        23: '460cm < daily snow depth ≤ 480cm',
        24: '480cm < daily snow depth ≤ 500cm',
        25: '500cm < daily snow depth ≤ 520cm',
        26: '520cm < daily snow depth ≤ 540cm',
        27: '540cm < daily snow depth ≤ 560cm',
        28: '560cm < daily snow depth ≤ 580cm',
        29: '580cm < daily snow depth ≤ 600cm',
        30: 'daily snow depth > 600cm',
    },
    undefined='Undefined snow depth (Code: %d)'
)

qzss_dcx_camf_d15_flood_severity = QzssDcrDefinition(
    {
        0: 'Minor Flooding - Minimal or no property damage, but possibly some public threat.',
        1: 'Moderate Flooding - Some inundation of structures and roads near stream. Some evacuations of people and/or transfer of property to higher elevations.',
        2: 'Major Flooding - Extensive inundation of structures and roads. Significant evacuations of people and/or transfer of property to higher elevations.',
        3: 'Record Flooding',
    },
    undefined='Undefined flood severity (Code: %d)'
)

qzss_dcx_camf_d16_lightning_intensity = QzssDcrDefinition(
    {
        0: 'LAL 1 - No thunderstorms',
        1: 'LAL 2 - Isolated thunderstorms. Light rain will occasionally reach the ground. Lightning is very infrequent, 1 to 5 cloud to ground strikes in a 5-minute period.',
        2: 'LAL 3 - Widely scattered thunderstorms. Light to moderate rain will reach the ground. Lightning is infrequent, 6 to 10 cloud to ground strikes in a 5-minute period.',
        3: 'LAL 4 - Scattered thunderstorms. Moderate rain is commonly produced Lightning is frequent, 11 to 15 cloud to ground strikes in a 5-minute period.',
        4: 'LAL 5 - Numerous thunderstorms. Rainfall is moderate to heavy. Lightning is frequent and intense, greater than 15 cloud to ground strikes in a 5-minute period.',
        5: 'LAL 6 - Dry lightning (same as LAL 3 but without rain). This type of lightning has the potential for extreme fire activity and is normally.',
    },
    undefined='Undefined lightning intensity (Code: %d)'
)

qzss_dcx_camf_d17_fog_level = QzssDcrDefinition(
    {
        0: 'Level 1 of 5: Slight fog or Mist. On land, object appear hazy or blurry. Road and rail traffic are unhindered. On sea, horizon cannot be seen. Lights and landmarks can be seen at working distances.',
        1: 'Level 2 of 5: Slight fog. On land, railroad traffic takes additional caution. On sea, Lights on passing vessel are generally not distinct at distances under 1 mile. Fog signals are sounded.',
        2: 'Level 3 of 5: Moderate fog. On land, rail and road traffic is obstructed. On sea, Lights on passing vessels are generally not distinct at distances under 1 mile. Fog signals are sounded. On river, navigation is unhindered but extra caution is required.',
        3: 'Level 4 of 5: Moderate fog. On land, rail and road traffic impeded. On sea, lights on ships and other vessels cannot be seen at distances of 4 miles or less. On river, navigation is suspended.',
        4: 'Level 5 of 5: Thick fog. On land, all traffic is impeded and totally disorganised. On sea, lights on ships and other vessels cannot be seen at distances of 4 miles or less. On river, navigation is suspended.',
    },
    undefined='Undefined fog level (Code: %d)'
)

qzss_dcx_camf_d18_drought_level = QzssDcrDefinition(
    {
        0: 'D1 – Moderate Drought – PDSI = -2.0 to -2.9. Some damage to crops, pastures. Streams, reservoirs, or wells low, some water shortages developing or imminent. Voluntary water-use restrictions requested.',
        1: 'D2 – Severe Drought – PDSI = -3.0 to -3.9. Crop or pasture losses likely. Water shortages common. Water restrictions imposed.',
        2: 'D3 – Extreme Drought – PDSI = -4.0 to -4.9. Major crop/pasture losses. Widespread water shortages or restrictions.',
        3: 'D4 – Exceptional Drought – PDSI = -5.0 or less. Exceptional and widespread crop/pasture losses. Shortages of water in reservoirs, streams, and wells creating water emergencies.',
    },
    undefined='Undefined drought level (Code: %d)'
)

qzss_dcx_camf_d19_avalanche_warning_level = QzssDcrDefinition(
    {
        0: '1 – Low. Generally stable conditions. Triggering is generally possible only from high additional loads in isolated areas of very steep, extreme terrain. Only small and medium natural avalanches are possible.',
        1: '2 – Moderate. Heightened avalanche conditions on specific terrain features. Triggering is possible, primarily from high additional loads, particularly on the indicated steep slopes. Very large natural avalanches are unlikely.',
        2: '3 – Considerable. Dangerous avalanche conditions Triggering is possible, even from low additional loads, particularly on the indicated steep slopes. In certain situations, some large, and in isolated cases very large natural avalanches are possible.',
        3: '4 – High. Very dangerous avalanche conditions. Triggering is likely, even from low additional loads, on many steep slopes. In some cases, numerous large and often very large natural avalanches can be expected.',
        4: '5 – Very high. Extraordinary avalanche conditions. Numerous very large and often extremely large natural avalanches can be expected, even in moderately steep terrain.',
    },
    undefined='Undefined avalanche warning level (Code: %d)'
)

qzss_dcx_camf_d20_ash_fall_amount_and_impact = QzssDcrDefinition(
    {
        0: 'Less than 1 mm ash thickness. Possible impact: Will act as an irritant to lungs and eyes. Possible minor damage to vehicles, houses and equipment caused by fine abrasive ash. Possible contamination of water supplies, particularly roof-fed tank supplies. Dust may affect road visibility and traction for an extended period.',
        1: '1-5 mm ash thickness. Possible impact: Will act as an irritant to lungs and eyes. Minor damage to vehicles, houses and equipment caused by fine abrasive ash. Possible contamination of water supplies, particularly roof-fed tank supplies. Electricity and water supplies may be cut or limited. Dust may affect road visibility and traction for an extended period. Roads may need to be cleared to reduce the dust nuisance and prevent storm-water systems from becoming blocked. Possible crop damage. Some livestock may be affected.',
        2: '5-100 mm ash thickness. Possible impact: Will act as an irritant to lungs and eyes. Damage to vehicles, houses and equipment caused by fine abrasive ash. Most buildings will support the ash load but weaker roof structures may collapse at 100 mm ash thickness, particularly if the ash is wet. Possible contamination of water supplies, particularly roof-fed tank supplies. Electricity and water supplies may be cut or limited. Road transport may be halted due to the build-up of ash on roads. Cars still working may soon stop due to clogging of air-filters. Rail transport may be forced to stop due to signal failure bought on by short circuiting if ash becomes wet. Likely crop damage. Most pastures will be killed by over 50 mm of ash. Some livestock may be affected.',
        3: '100-300 mm ash thickness. Possible impact: Will act as an irritant to lungs and eyes. Damage to vehicles, houses and equipment caused by fine abrasive ash. Buildings that are not cleared of ash will run the risk of roof collapse, especially large flat roofed structures and if ash becomes wet. Possible contamination of water supplies, particularly roof-fed tank supplies. Electricity and water supplies may be cut or limited. Road transport may be halted due to the build-up of ash on roads. Cars still working may soon stop due to clogging of air-filters. Rail transport may be forced to stop due to signal failure bought on by short circuiting if ash becomes wet. Likely crop damage. Most pastures will be killed by over 50 mm of ash. Some livestock may be affected.',
        4: '> 300 mm ash thickness. Possible impact: Will act as an irritant to lungs and eyes. Damage to vehicles, houses and equipment caused by fine abrasive ash. Buildings that are not cleared of ash will run the risk of roof collapse, especially large flat roofed structures and if ash becomes wet. Possible contamination of water supplies, particularly roof-fed tank supplies. Electricity and water supplies may be cut or limited. Road unusable until cleared. Rail transport may be forced to stop due to signal failure bought on by short circuiting if ash becomes wet. Heavy kill of vegetation. Livestock and other animals killed or heavily distressed.',
    },
    undefined='Undefined ash fall amount and impact (Code: %d)'
)

qzss_dcx_camf_d21_geomagnetic_scale = QzssDcrDefinition(
    {
        0: 'G1 - Minor. Power systems: Weak power grid fluctuations can occur. Spacecraft operations: Minor impact on satellite operations possible. Other systems: Migratory animals are affected at this and higher levels; aurora is commonly visible at high latitudes.',
        1: 'G2 - Moderate. Power systems: High-latitude power systems may experience voltage alarms, long-duration storms may cause transformer damage. Spacecraft operations: Corrective actions to orientation may be required by ground control; possible changes in drag affect orbit predictions. Other systems: HF radio propagation can fade at higher latitudes, and aurora has been seen as low as 55° geomagnetic lat.',
        2: 'G3 - Strong. Power systems: Voltage corrections may be required, false alarms triggered on some protection devices. Spacecraft operations: Surface charging may occur on satellite components, drag may increase on low-Earth-orbit satellites, and corrections may be needed for orientation problems. Other systems: Intermittent satellite navigation and low-frequency radio navigation problems may occur, HF radio may be intermittent, and aurora has been seen as low as 50° geomagnetic lat.',
        3: 'G4 - Severe. Power systems: Possible widespread voltage control problems and some protective systems will mistakenly trip out key assets from the grid. Spacecraft operations: May experience surface charging and tracking problems, corrections may be needed for orientation problems. Other systems: Induced pipeline currents affect preventive measures, HF radio propagation sporadic, satellite navigation degraded for hours, low-frequency radio navigation disrupted, and aurora has been seen as low as 45° geomagnetic lat.',
        4: 'G5 - Extreme. Power systems: Widespread voltage control problems and protective system problems can occur; some grid systems may experience complete collapse or blackouts. Transformers may experience damage. Spacecraft operations: May experience extensive surface charging, problems with orientation, uplink/downlink and tracking satellites. Other systems: Pipeline currents can reach hundreds of amps, HF (high frequency) radio propagation may be impossible in many areas for one to two days, satellite navigation may be degraded for days, low-frequency radio navigation can be out for hours, and aurora has been seen as low 40° geomagnetic lat.',
    },
    undefined='Undefined geomagnetic scale (Code: %d)'
)

qzss_dcx_camf_d22_terrorism_threat_level = QzssDcrDefinition(
    {
        0: 'Very low threat level. A violent act of terrorism is highly unlikely. Measures are in place to keep the population safe.',
        1: 'Low threat level. A violent act of terrorism is possible but unlikely. Measures are in place to keep the population safe.',
        2: 'Medium threat level. A violent act of terrorism could occur. Additional measures are in place to keep the population safe.',
        3: 'High threat level. A violent act of terrorism is likely. Heightened measures are in place to keep the population safe.',
        4: 'Critical threat level. A violent act of terrorism is highly likely and could occur imminently. Exceptional measures are in place to keep the population safe.',
    },
    undefined='Undefined geomagnetic scale (Code: %d)'
)

qzss_dcx_camf_d23_fire_risk_level = QzssDcrDefinition(
    {
        0: 'Danger level 1/5 (low or none danger). Small fires cannot be entirely ruled out, but require a high energy input. Lightning hardly ever causes a fire. Rate of spread: Generally slow. Characteristics: Surface or crawling fires, crowns of trees are not affected, topsoil does not burn. Fire-fighting: Forest fire is easy to extinguish. Behaviour: Do not carelessly discard cigarettes, tobacco products or lighters.',
        1: 'Danger level 2/5 (moderate danger). Local fires can start spontaneously. Lightning only rarely causes a conflagration. Rate of spread: Slow to medium. Characteristics: Surface or crawling fires, crowns of trees are rarely affected, topsoil is burnt a little or not at all. Fire-fighting: Forest fire is ordinarily easy to extinguish. Behaviour: Do not carelessly discard cigarettes, tobacco products or lighters. Always watch barbecue fires and immediately extinguish stray sparks.',
        2: 'Danger level 3/5 (considerable danger): Burning matches and flying sparks from barbecue fires can ignite a fire. Lightning can also trigger widespread fires. Rate of spread: High in open terrain, medium in the forest. Characteristics. Topsoil is partly burnt; individual crown fires are possible. Fire-fighting: Forest fire can be extinguished only by experts using modern equipment. Behaviour: Light barbecue fires only in existing fire places. Always watch the fire and immediately extinguish stray sparks.',
        3: 'Danger level 4/5 (high danger). Burning matches, flying sparks from barbecue fires and lightning will very probably ignite a fire. Rate of spread: High, including in forests. Characteristics: Intense surface fires can ignite the crowns of individual trees, spotting is possible, burning topsoil. Fire-fighting: Forest fire is difficult to extinguish and commands extensive resources. Behaviour: As a general rule, do not make any fires outdoors. Permanent fire places (concreted base) in locations designated by the authorities can be used with the utmost caution. Do not make fires in strong winds.',
        4: 'Danger level 5/5 (very high danger). Fires can start at any time. Rate of spread: Very high over a long period. Characteristics: Very intense burning, extensive crown fires, long-distance spotting. Fire-fighting: Forest fire is virtually impossible to extinguish. Behaviour: Do not make any fires outdoors. Follow the instructions and observe the fire bans imposed by the local authorities.',
    },
    undefined='Undefined fire risk level (Code: %d)',
)

qzss_dcx_camf_d24_water_quality = QzssDcrDefinition(
    {
        0: 'Excellent water quality',
        1: 'Good water quality',
        2: 'Poor water quality',
        3: 'Very poor water quality',
        4: 'Suitable for drinking purposes',
        5: 'Unsuitable for drinking purpose',
    },
    undefined='Undefined water quality (Code: %d)'
)

qzss_dcx_camf_d25_uv_index = QzssDcrDefinition(
    {
        0: 'Index 0 - 2 Low. No protection needed. You can safely stay outside using minimal sun protection.',
        1: 'Index 3/11 Moderate. Protection needed. Seek shade during late morning through mid-afternoon. When outside, generously apply broad-spectrum SPF-15 or higher sunscreen on exposed skin, and wear protective clothing, a wide-brimmed hat, and sunglasses.',
        2: 'Index 4/11 Moderate. Protection needed. Seek shade during late morning through mid-afternoon. When outside, generously apply broad-spectrum SPF-15 or higher sunscreen on exposed skin, and wear protective clothing, a wide-brimmed hat, and sunglasses.',
        3: 'Index 5/11 High. Protection needed. Seek shade during late morning through mid-afternoon. When outside, generously apply broad-spectrum SPF-15 or higher sunscreen on exposed skin, and wear protective clothing, a wide-brimmed hat, and sunglasses.',
        4: 'Index 6/11 High. Protection needed. Seek shade during late morning through mid-afternoon. When outside, generously apply broad-spectrum SPF-15 or higher sunscreen on exposed skin, and wear protective clothing, a wide-brimmed hat, and sunglasses.',
        5: 'Index 7/11 High. Protection needed. Seek shade during late morning through mid-afternoon. When outside, generously apply broad-spectrum SPF-15 or higher sunscreen on exposed skin, and wear protective clothing, a wide-brimmed hat, and sunglasses.',
        6: 'Index 8/11 Very high. Extra protection needed. Be careful outside, especially during late morning through mid-afternoon. If your shadow is shorter than you, seek shade and wear protective clothing, a wide-brimmed hat, and sunglasses, and generously apply a minimum of SPF-15, broad-spectrum sunscreen on exposed skin.',
        7: 'Index 9/11 Very high. Extra protection needed. Be careful outside, especially during late morning through mid-afternoon. If your shadow is shorter than you, seek shade and wear protective clothing, a wide-brimmed hat, and sunglasses, and generously apply a minimum of SPF-15, broad-spectrum sunscreen on exposed skin.',
        8: 'Index 10/11 Extreme. Extra protection needed. Be careful outside, especially during late morning through mid-afternoon. If your shadow is shorter than you, seek shade and wear protective clothing, a wide-brimmed hat, and sunglasses, and generously apply a minimum of SPF-15, broad-spectrum sunscreen on exposed skin.',
        9: 'Index 11/11 Extreme. Extra protection needed. Be careful outside, especially during late morning through mid-afternoon. If your shadow is shorter than you, seek shade and wear protective clothing, a wide-brimmed hat, and sunglasses, and generously apply a minimum of SPF-15, broad-spectrum sunscreen on exposed skin.',
    },
    undefined='Undefined UV index (Code: %d)'
)

qzss_dcx_camf_d26_number_of_cases_per_100000_inhabitants = QzssDcrDefinition(
    {
        0: '0 - 9',
        1: '10 - 20',
        2: '21 - 50',
        3: '51 - 70',
        4: '71 - 100',
        5: '101- 125',
        6: '126 - 150',
        7: '151 - 175',
        8: '176 - 200',
        9: '201 - 250',
        10: '251 - 300',
        11: '301 - 350',
        12: '351 - 400',
        13: '401 - 450',
        14: '451 - 500',
        15: '501 - 750',
        16: '751 - 1000',
        17: '> 1000',
        18: '> 2000',
        19: '> 3000',
        20: '> 5000',
    },
    undefined='Undefined number of cases per 100000 inhabitants (Code: %d)'
)

qzss_dcx_camf_d27_noise_range = QzssDcrDefinition(
    {
        0: '40 < dB ≤ 45',
        1: '45 < dB ≤ 50',
        2: '50 < dB ≤ 60',
        3: '60 < dB ≤ 70',
        4: '70 < dB ≤ 80 (loud)',
        5: '80 < dB ≤ 90 (very loud)',
        6: '90 < dB ≤ 100 (very loud)',
        7: '100 <dB ≤ 110 (very loud)',
        8: '110 < dB ≤ 120 (extremely loud)',
        9: '120 < dB ≤ 130 (extremely loud)',
        10: '130 < dB ≤ 140 (threshold of pain)',
        11: 'dB > 140 (pain)',
    },
    undefined='Undefined noise range (Code: %d)'
)

qzss_dcx_camf_d28_air_quality_index = QzssDcrDefinition(
    {
        0: 'Index value 0 - 50. Good. Green. Advisory: None.',
        1: 'Index value 51 - 100. Moderate. Yellow. Unusually sensitive individuals should consider limiting prolonged outdoor exertion.',
        2: 'Index 101 - 150. Unhealthy for sensitive groups. Orange. Children, active adults and people with respiratory disease, such as asthma, should limit prolonged outdoor exertion',
        3: 'Index 151 - 200. Unhealthy. Red. Children, active adults and people with respiratory disease, such as asthma, should limit prolonged outdoor exertion. Everyone else should limit prolonged outdoor exertion.',
        4: 'Index 201 - 300. Very unhealthy. Purple. Children, active adults and people with respiratory disease, such as asthma, should limit prolonged outdoor exertion. Everyone else should limit outdoor exertion.',
        5: 'Index 301 - 500. Hazardous.Brown.Everyone should avoid all physical activity outdoor.',
    },
    undefined='Undefined air quality index (Code: %d)'
)

qzss_dcx_camf_d29_outage_estimated_duration = QzssDcrDefinition(
    {
        0: '0 < duration < 30 min',
        1: '30 min ≤ duration < 45 min',
        2: '45 min ≤ duration < 1 h',
        3: '1 h ≤ duration < 1h 30min',
        4: '1h 30min ≤ duration < 2 h',
        5: '2 h ≤ duration < 3 h',
        6: '3 h ≤ duration < 4 h',
        7: '4 h ≤ duration < 5 h',
        8: '5 h ≤ duration < 10 h',
        9: '10 h ≤ duration < 24 h',
        10: '24 h ≤ duration < 2 days',
        11: '2 days ≤ duration < 7 days',
        12: '7 days ≤ duration',
    },
    undefined='Undefined outage estimated duration (Code: %d)'
)

qzss_dcx_camf_d30_nuclear_event_scale = QzssDcrDefinition(
    {
        0: 'Unknown',
        1: 'Level 0/7. Deviation. No safety significance',
        2: 'Level 1/7. Anomaly. Overexposure in excess of statutory annual limits. Minor problems with safety components with significant defence-in-depth remaining. Low activity lost or stolen radioactive source, device, or transport package.',
        3: 'Level 2/7. Incident. Impact on people and environment: Exposure of the public in excess of 10 mSv. Exposure of workers in excess of the statutory annual limits. Impact on radiological barriers and control: Radiation levels in an operating area of more than 50 mSv/h. Significant contamination within the facility into an area not expected by design. Possible cause: Significant failures in safety provisions but with no actual consequences. Found highly radioactive sealed orphan source, device or transport package with safety provisions intact. Inadequate packaging of a highly radioactive sealed source.',
        4: 'Level 3/7. Serious incident. Impact on people and environment: Exposure in excess of ten times the statutory annual limit for workers. Non-lethal deterministic health effect (e.g., burns) from radiation. Impact on radiological barriers and control: Exposure rates of more than 1 Sv/h in an operating area. Severe contamination in an area not expected by design, with a low probability of significant public exposure. Possible cause: Near-accident at a nuclear power plant with no safety provisions remaining. Lost or stolen highly radioactive sealed source. Misdelivered highly radioactive sealed source without adequate procedures in place to handle it.',
        5: 'Level 4/7. Accident with local consequences. Impact on people and environment: Minor release of radioactive material unlikely to result in implementation of planned countermeasures other than local food controls. Impact on radiological barriers and control: Release of significant quantities of radioactive material within an installation with a high probability of significant public exposure.',
        6: 'Level 5/7. Accident with wider consequences. Impact on people and environment: Limited release of radioactive material likely to require implementation of some planned countermeasures. Major health impact from radiation is likely. Impact on radiological barriers and control: Severe damage to reactor core. Release of large quantities of radioactive material within an installation with a high probability of significant public exposure. This could arise from a major criticality accident or fire.',
        7: 'Level 6/7. Serious accident. Impact on people and environment: Significant release of radioactive material likely to require implementation of planned countermeasures.',
        8: 'Level 7/7. Major accident. Impact on people and environment: Major release of radioactive material with widespread health and environmental effects requiring implementation of planned and extended countermeasures.',
    },
    undefined='Undefined nuclear event scale (Code: %d)'
)

qzss_dcx_camf_d31_chemical_hazard_type = QzssDcrDefinition(
    {
        0: 'Explosives',
        1: 'Flammable gases',
        2: 'Flammable aerosols and aerosols',
        3: 'Oxidizing gases',
        4: 'Gases under pressure',
        5: 'Flammable liquids',
        6: 'Flammable solids',
        7: 'Self-reactive substance/mixture',
        8: 'Pyrophoric liquids. Pyrophoric materials are often water-reactive as well and will ignite when they contact water or humid air.',
        9: 'Pyrophoric solids. Pyrophoric materials are often water-reactive as well and will ignite when they contact water or humid air.',
        10: 'Self-heating substance/mixture',
        11: 'Water-reactive - emits flammable gases',
        12: 'Oxidising liquids',
        13: 'Oxidising solids',
        14: 'Organic peroxides',
        15: 'Corrosive to metals',
    },
    undefined='Undefined chemical hazard type (Code: %d)'
)

qzss_dcx_camf_d32_biohazard_level = QzssDcrDefinition(
    {
        0: 'Biohazard Level 1/4: Often pertains to agents that include viruses and bacteria, this biosafety level requires minimal precaution, such as wearing face masks and maintaining no close contact. The biological hazard examples in the first level include E. coli and other non-infectious bacteria.',
        1: 'Biohazard Level 2/4: Usually causing severe diseases to humans, the second level classifies agents that can be transmitted through direct contact with infected materials. HIV and hepatitis B are some biological hazard examples that pose moderate risks to humans.',
        2: 'Biohazard Level 3/4: Mainly through respiratory transmission, pathogens that are highly likely to become airborne can cause serious or lethal diseases to humans. Mycobacterium tuberculosis, the bacteria that causes tuberculosis, is an example of a level-3 biohazard.',
        3: 'Biohazard Level 4/4: Extremely dangerous pathogens that expose humans to life-threatening diseases, the fourth and last level requires workers to utilise maximum protection and containment. Some biological hazard examples are the Ebola virus and the Lassa virus.',
    },
    undefined='Undefined biohazard level (Code: %d)'
)

qzss_dcx_camf_d33_biohazard_type = QzssDcrDefinition(
    {
        0: "Biological agents. These include bacteria, viruses, parasites, and fungi (such as yeasts and molds).",
        1: "Biotoxins. These refer to a group of substances with a biological origin that are toxic and poisonous to humans. Often, biotoxins are produced by plants, bacteria, insects, or certain animals, among others.Continuous exposure to these may cause, at the very least, a series of inflammatory reactions throughout the body.",
        2: "Blood and blood products. While blood isn't considered a biological hazard, it can still bring potential risks if it's contaminated or its source is in any way infected. Also, blood products such as red blood cells, white blood cells, plasma, tissues, and platelets are also hazardous if not properly handled.",
        3: "Environmental specimens. Generally, these refer to plants, soil, or water that potentially contain biological agents(include bacteria, viruses, parasites, and fungi) and biotoxins.",
    },
    undefined='Undefined biohazard type (Code: %d)'
)

qzss_dcx_camf_d34_explosive_hazard_type = QzssDcrDefinition(
    {
        0: 'PE1 - Mass explosion hazard in which the entire body of explosives explodes as one.',
        1: 'PE2 - Serious projectile hazard but does not have a mass explosion hazard.',
        2: 'PE3 - Fire hazard and either a minor blast hazard or a minor projection hazard, or both, but does not have a mass explosion hazard. Explosives which give rise to considerable radiant heat or which burn to produce a minor blast or projection hazard.',
        3: 'PE4 - Fire or slight explosion hazard, or both, with only local effect. Explosives which present only a low hazard in the event of ignition or initiation, where no significant blast or projection of fragments of appreciable size or range is expected.',
    },
    undefined='Undefined explosive hazard type (Code: %d)'
)

qzss_dcx_camf_d35_infection_type = QzssDcrDefinition(
    {
        0: "Anthrax",
        1: "Avian influenza in humans",
        2: "Botulism",
        3: "Brucellosis",
        4: "Campylobacteriosis",
        5: "Chikungunya virus disease",
        6: "Chlamydia infections",
        7: "Cholera",
        8: "COVID - 19",
        9: "Creutzfeldt - Jakob Disease - variant(vCJD)",
        10: "Cryptosporidiosis",
        11: "Dengue",
        12: "Diphtheria",
        13: "Echinococcosis",
        14: "Giardiasis",
        15: "Gonorrhoea",
        16: "Hepatitis A",
        17: "Hepatitis B",
        18: "Hepatitis C",
        19: "HIV infection and AIDS",
        20: "Infections with haemophilus influenza group B",
        21: "Influenza including Influenza A(H1N1)",
        22: "Invasive meningococcal disease",
        23: "Invasive pneumococcal disease",
        24: "Legionnaries' disease",
        25: "Leptospirosis",
        26: "Listeriosis",
        27: "Lyme neuroborreliosis",
        28: "Malaria",
        29: "Measles",
        30: "Meningoccocal disease, invasive",
        31: "Mumps",
        32: "Pertussis",
        33: "Plague",
        34: "Pneumoccocal invasive diseases",
        35: "Poliomyelitis",
        36: "Q fever",
        37: "Rabies",
        38: "Rubella",
        39: "Rubella, congenital",
        40: "Salmonellosis",
        41: "Severe Acute Respiratory Syndrome (SARS)",
        42: "Shiga toxin /verocytotoxin -producing Escherichia coli (STEC/VTEC)",
        43: "Shigellosis",
        44: "Smallpox",
        45: "Syphilis",
        46: "Syphilis, congenital",
        47: "Tetanus",
        48: "Tick-borne encephalitis",
        49: "Toxoplasmosis, congenital",
        50: "Trichinellosis",
        51: "Tuberculosis",
        52: "Tularaemia",
        53: "Typhoid and paratyphoid fevers",
        54: "Viral haemorrhagic fevers",
        55: "West Nile virus infection",
        56: "Yellow fever",
        57: "Yersinosis",
        58: "Zika virus disease",
        59: "Zika virus disease, congenital",
        60: "Nosocomial infections",
        61: "Antimicrobial resistance",
        62: "unidentified infection",
    },
    undefined='Undefined infection type (Code: %d)'
)

qzss_dcx_camf_d36_typhoon_category = QzssDcrDefinition(
    {
        0: 'Scale 1 and Intensity 1',
        1: 'Scale 1 and Intensity 2',
        2: 'Scale 1 and Intensity 3',
        3: 'Scale 2 and Intensity 1',
        4: 'Scale 2 and Intensity 2',
    },
    undefined='Undefined typhoon category (Code: %d)'
)



#!/usr/bin/env python3

import argparse
import logging
import socket
from pprint import pformat

from ..qzss_dcr_lib.interface import decode

logger = logging.getLogger(__name__)


class Receiver:
    def __init__(self, bind_addr='::', bind_port=2112, bind_iface='any', address_family=socket.AF_UNSPEC):
        self.addr_info = socket.getaddrinfo(bind_addr, bind_port,
                                            address_family,
                                            socket.SOCK_DGRAM,
                                            socket.IPPROTO_UDP)[0]
        if bind_iface == 'any':
            self.bind_iface = None
        else:
            self.bind_iface = (bind_iface + '\0').encode()

    @staticmethod
    def default_handler(report):
        logger.info('- - - - - - - - - - - - - - - - - - - - - - - - - - - -\n' + pformat(report.get_params()) + '\n')

    def start(self, callback=None, callback_args=(), callback_kwargs={}):
        callback = callback or self.default_handler
        with socket.socket(self.addr_info[0], self.addr_info[1]) as sock:
            if self.bind_iface is not None:
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_BINDTODEVICE, self.bind_iface)
            sock.bind(self.addr_info[-1])
            while True:
                data = sock.recvfrom(256)
                payload = data[0]
                callback(decode(payload, 'net'),
                         *callback_args, **callback_kwargs)


def simple_handler(report):
    logger.info('\n' + str(report) + '\n')


def main():
    logging.basicConfig(level=logging.INFO,
                        format='%(asctime)s - %(levelname)s - %(message)s')
    parser = argparse.ArgumentParser(description='azarashi network receiver',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-b', '--bind-addr', help="address to bind", type=str, default='::')
    parser.add_argument('-p', '--bind-port', help='port to bind', type=int, default=2112)
    parser.add_argument('-i', '--bind-iface', help="iface to bind", type=str, default='any')
    parser.add_argument('-v', '--verbose', help="verbose mode", action='store_true')
    args = parser.parse_args()
    recver = Receiver(args.bind_addr, args.bind_port)
    if args.verbose:
        recver.start()
    else:
        recver.start(callback=simple_handler)


if __name__ == '__main__':
    exit(main())



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcx_camf_a11_japanese_library_en = QzssDcrDefinition(
    {
        0: "",
        1: "Take the best immediate action to save your life.",
        126: "This is a test message for DCX.",
        127: "Take the best immediate action to save your life.",
        128: "Missile launched, missile launched. It is believed that a missile was launched. Please take shelter inside buildings or underground.",
        129: "Missile passed, missile passed. It is believed that the previous missile has passed over the area. The call for evacuation will be canceled. If you find any suspicious object, please stay away from it and inform the police or the fire department immediately.",
        130: "It is believed that the previous missile has dropped in the sea. The call for evacuation will be canceled. If you find any suspicious object, please stay away from it and inform the police or the fire department immediately.",
        131: "It is believed that the previous missile will not come to Japan. The call for evacuation will be canceled.",
        132: "Take shelter immediately, take shelter immediately. Please take shelter inside buildings or underground. It is believed that a missile will drop around this area. Please take shelter immediately.",
        133: "The previous missile has been intercepted and destroyed. There is a possibility of pieces of the destroyed missile dropping. We will keep you informed. Please stay indoors for shelter.",
        134: "Missile dropped, missile dropped. It is believed that a missile has dropped around this area. We will keep you informed. Please stay indoors for shelter.",
        135: "It is believed that the previous missile will not drop in Japan. The call for evacuation will be canceled.",
        136: "This is a test message for J-Alert.",
        255: "Take immediate action to save your life.",
        256: "Stay.",
        257: "Stay. Under/inside a solid structure.",
        258: "Stay. 3rd floor or higher.",
        259: "Stay. Underground.",
        260: "Stay. Mountain.",
        261: "Stay. Water area.",
        262: "Stay. Building where chemicals are handled, such as a factory.",
        263: "Stay. Cliffs and areas at risk of collapse.",
        512: "Move to/toward",
        513: "Move to/toward Under/inside a solid structure.",
        514: "Move to/toward 3rd floor or higher.",
        515: "Move to/toward Underground.",
        516: "Move to/toward Mountain.",
        517: "Move to/toward Water area.",
        518: "Move to/toward Building where chemicals are handled, such as a factory.",
        519: "Move to/toward Cliffs and areas at risk of collapse.",
        768: "Keep away from",
        769: "Keep away from Under/inside a solid structure.",
        770: "Keep away from 3rd floor or higher.",
        771: "Keep away from Underground.",
        772: "Keep away from Mountain.",
        773: "Keep away from Water area.",
        774: "Keep away from Building where chemicals are handled, such as a factory.",
        775: "Keep away from Cliffs and areas at risk of collapse.",
    },
    undefined = "Undefined guidance to react instruction in the japanese library. (Code: %d)"
)


qzss_dcx_camf_a11_japanese_library_ja = QzssDcrDefinition(
    {
        0: "",
        1: "直ちに命を守るための最善の行動を。",
        126: "これは、DCX のテストです。",
        127: "直ちに命を守るための最善の行動を。",
        128: "ミサイル発射。ミサイル発射。ミサイルが発射されたものとみられます。建物の中、又は地下に避難して下さい。",
        129: "ミサイル通過。ミサイル通過。先程のミサイルは通過したものとみられます。避難の呼びかけを解除します。不審なものには決して近寄らず直ちに警察や消防などに連絡して下さい。",
        130: "先程のミサイルは、海に落下したものとみられます。避難の呼びかけを解除します。不審なものには決して近寄らず直ちに警察や消防などに連絡して下さい。",
        131: "先程のミサイルは、我が国には飛来しないものとみられます。避難の呼びかけを解除します。",
        132: "直ちに避難。直ちに避難。直ちに建物の中、又は地下に避難して下さい。ミサイルが、周辺に落下するものとみられます。直ちに避難して下さい。",
        133: "先程のミサイルは、迎撃により破壊されました。ミサイルの破片の落下の可能性があります。続報を伝達しますので、引き続き屋内に避難して下さい。",
        134: "ミサイル落下。ミサイル落下。ミサイルが、周辺に落下したものとみられます。続報を伝達しますので、引き続き屋内に避難して下さい。",
        135: "先程のミサイルは、我が国には落下しないものとみられます。避難の呼びかけを解除します。",
        136: "これは、Jアラートのテストです。",
        255: "直ちに命を守るための最善の行動を。",
        256: "留まれ。",
        257: "留まれ。頑丈なものの下/中。",
        258: "留まれ。3階以上。",
        259: "留まれ。地下。",
        260: "留まれ。山。",
        261: "留まれ。水場。",
        262: "留まれ。工場等化学系を取扱う建物。",
        263: "留まれ。崖等崩れやすい場所。",
        512: "向かえ。",
        513: "向かえ。頑丈なものの下/中。",
        514: "向かえ。3階以上。",
        515: "向かえ。地下。",
        516: "向かえ。山。",
        517: "向かえ。水場。",
        518: "向かえ。工場等化学系を取扱う建物。",
        519: "向かえ。崖等崩れやすい場所。",
        768: "離れろ。",
        769: "離れろ。頑丈なものの下/中。",
        770: "離れろ。3階以上。",
        771: "離れろ。地下。",
        772: "離れろ。山。",
        773: "離れろ。水場。",
        774: "離れろ。工場等化学系を取扱う建物。",
        775: "離れろ。崖等崩れやすい場所。",
    },
    undefined = "避難情報(コード番号：%d)"
)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_satellite_prn = QzssDcrDefinition(
    {
        55: "PRN183",
        56: "PRN184",
        57: "PRN185",
        58: "PRN186",
        61: "PRN189",
    },
    undefined="Undefined PRN (Code: %d)"
)



from .hex_qzss_dcr_decoder import *
from .net_qzss_dcr_decoder import *
from .nmea_qzss_dcr_decoder import *
from .qzss_dcr_decoder import *
from .qzss_dcr_decoder_base import *
from .qzss_dcr_decoder_jma import *
from .qzss_dcr_decoder_jma_ash_fall import *
from .qzss_dcr_decoder_jma_common import *
from .qzss_dcr_decoder_jma_earthquake_early_warning import *
from .qzss_dcr_decoder_jma_flood import *
from .qzss_dcr_decoder_jma_hypocenter import *
from .qzss_dcr_decoder_jma_marine import *
from .qzss_dcr_decoder_jma_northwest_pacific_tsunami import *
from .qzss_dcr_decoder_jma_seismic_intensity import *
from .qzss_dcr_decoder_jma_tsunami import *
from .qzss_dcr_decoder_jma_typhoon import *
from .qzss_dcr_decoder_jma_volcano import *
from .qzss_dcr_decoder_jma_weather import *
from .qzss_dcx_decoder import *
from .ublox_qzss_dcr_decoder import *



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_preamble = QzssDcrDefinition(
    {
        0x53: "A",  # 01010011
        0x9A: "B",  # 10011010
        0xC6: "C",  # 11000110
    },
    undefined="Undefined Preamble (Code: %d)"
)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcx_camf_c10_guidance_library_for_second_ellipse_code = QzssDcrDefinition(
    {
        0: "IC-C-01",
        1: "IC-C-02",
        2: "IC-C-03",
        3: "IC-C-04",
        4: "IC-C-05",
        5: "IC-C-06",
        6: "IC-C-07",
        7: "IC-C-08",
        8: "IC-C-09",
        9: "IC-C-10",
        10: "IC-C-11",
        11: "IC-C-12",
        12: "IC-C-13",
        13: "IC-C-14",
        14: "IC-C-15",
        15: "IC-C-16",
        16: "IC-C-17",
        30: "IC-C-31",
        31: "IC-C-32",
    },
    undefined='IC-C-UNDEFINED (Code: %d)'
)

qzss_dcx_camf_c10_guidance_library_for_second_ellipse = QzssDcrDefinition(
    {
        0: "",
        1: "Prepare for evacuation. Take only the essentials with you, especially ID cards, passport, credit cards and cash. Evacuate only after the instruction of the emergency authorities.",
        2: "Prepare emergency food and relief material: Check and restock your equipment and supplies of water, food, medicine, cash and batteries.",
        3: "Be prepared to protect yourself and your property. Flooding of properties and transport networks is expected. Disruption to power, communications and water supplies are possible. Evacuation may be required. Dangerous driving conditions due to reduced visibility and aquaplaning.",
        4: "Have iodine tablets ready. DO NOT take the iodine tablets now. If this becomes necessary, we will inform you in good time.",
        5: "Keep your smartphone charged to be able to receive further instructions and information.",
        6: "Avoid using lifts.",
        7: "Avoid the danger area.",
        8: "Avoid driving.",
        9: "Rescue operation under process by security forces and emergency services. Avoid moving to facilitate security and emergency actions.",
        10: "Check with the weather services and local authorities for additional information.",
        11: "Find out the location of the information points set up by the authorities on official channels (radio, internet, TV, social networks...).",
        12: "Sensitive or vulnerable people should not go out unless they must.",
        13: "Protect the most vulnerable and hear from your loved ones. Be aware of their special needs and support, as required. If you notice distressed or vulnerable persons, contact the emergency services. Provide first aid if necessary but do not put yourself in any danger.",
        14: "Pay attention to announcements made by the police, fire brigade and by officials.",
        15: "Stay aware, keep listening to official instructions broadcast on the radio, television, websites and social networks pages.",
        16: "Only make phone calls in serious emergencies to avoid overloading the mobile network.",
        30: "This is only a test. You do not have to take any action or to adopt any particular sheltering behaviour.",
        31: "Conditions have improved and are no longer expected to meet alert criteria.",
    },
    undefined='Undefined instruction for second ellipse. (Code: %d)'
)



import datetime

from .hex_interface import hex_qzss_dcr_message_extractor
from .nmea_interface import nmea_qzss_dcr_message_extractor
from .ublox_interface import ublox_qzss_dcr_message_extractor
from ..decoder import HexQzssDcrDecoder
from ..decoder import NetQzssDcrDecoder
from ..decoder import NmeaQzssDcrDecoder
from ..decoder import UBloxQzssDcrDecoder
from ..exception import QzssDcrDecoderException

caches = {}
cache_size = 256
cache_expiration = 3600 * 24


def decode(msg, msg_type='nmea'):
    if not msg:
        raise EOFError('Encountered EOF')

    if msg_type == 'hex':
        return HexQzssDcrDecoder(msg).decode()
    elif msg_type == 'net':
        return NetQzssDcrDecoder(msg).decode()
    elif msg_type == 'nmea' or msg_type == 'spresense':
        return NmeaQzssDcrDecoder(msg).decode()
    elif msg_type == 'ublox':
        return UBloxQzssDcrDecoder(msg).decode()
    else:
        raise QzssDcrDecoderException(f'Unknown Message Type: {msg_type}')


def decode_stream(stream,  # do not decode one stream in parallel!
                  msg_type='nmea',
                  callback=None,
                  callback_args=(),
                  callback_kwargs={},
                  unique=False,
                  ignore_dcr=False,
                  ignore_dcx=True):
    for existing_stream in caches.copy().keys():
        if existing_stream.closed:
            try:
                caches.pop(existing_stream)  # discards the garbage to prevent memory leaks
            except KeyError:  # might happen during race conditions
                pass

    if unique:
        cache = caches.get(stream)
        if cache is None:
            cache = []

    if msg_type == 'hex':
        if callable(getattr(stream, 'readline', None)):
            extractor = hex_qzss_dcr_message_extractor
            reader = stream.readline
            reader_kwargs = {}
        else:
            raise QzssDcrDecoderException(f'readline() does not exist: {type(stream)}')
    elif msg_type == 'nmea' or msg_type == 'spresense':
        if callable(getattr(stream, 'readline', None)):
            extractor = nmea_qzss_dcr_message_extractor
            reader = stream.readline
            reader_kwargs = {}
        else:
            raise QzssDcrDecoderException(f'readline() does not exist: {type(stream)}')
    elif msg_type == 'ublox':
        if callable(getattr(stream, 'read1', None)):
            extractor = ublox_qzss_dcr_message_extractor
            reader = stream.read1
            reader_kwargs = {}
        elif hasattr(stream, 'buffer') and callable(getattr(stream.buffer, 'read1', None)):
            extractor = ublox_qzss_dcr_message_extractor
            reader = stream.buffer.read1
            reader_kwargs = {}
        elif callable(getattr(stream, 'read', None)):
            extractor = ublox_qzss_dcr_message_extractor
            reader = stream.read
            reader_kwargs = {'size': 1}
        else:
            raise QzssDcrDecoderException(f'Neither read() nor read1() exists: {type(stream)}')
    else:
        raise QzssDcrDecoderException(f'Unknown Message Type: {msg_type}')

    while True:
        msg = extractor(reader, reader_kwargs=reader_kwargs)
        report = decode(msg, msg_type)

        if report.message_type == 'DCR':
            if ignore_dcr is True:
                continue
        elif report.message_type == 'DCX':
            if ignore_dcx is True:
                continue
        else:  # unknown message type
            continue

        if unique:
            if report in cache:
                freshness = (datetime.datetime.now() - report.timestamp).total_seconds()
                if freshness > cache_expiration:  # the cache is rotten
                    fire = True
                else:  # the cache is fresh
                    fire = False
                cache.remove(report)
            else:
                fire = True

            cache = cache[-(cache_size - 1):] + [report]
            caches.update({stream: cache})

            if fire is False:
                continue

        if callback is None:
            return report
        callback(report, *callback_args, **callback_kwargs)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_jma_tsunami_warning_code = QzssDcrDefinition(
    {
        1: "津波なし",
        2: "警報解除",
        3: "津波警報",
        4: "大津波警報",
        5: "大津波警報：発表",
        15: "その他の警報",
        # "NN*": "警報(コード番号：NN)",
    },
    undefined="警報(コード番号：%d)"
)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcx_camf_a5_severity = QzssDcrDefinition(
    {
        0: 'Unknown',
        1: 'Moderate - Possible threat to life or property',
        2: 'Severe - Significant threat to life or property',
        3: 'Extreme - Extraordinary threat to life or property',
    },
    undefined='Undefined Severity (Code: %d)'
)


import setuptools

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='azarashi',
    version='0.15.1',
    description='QZSS DCR Decoder',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/nbtk/azarashi',
    author='nbtk',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
    ],
    packages=setuptools.find_packages(),
    install_requires=[],
    python_requires='>=3.9',
    entry_points={
        'console_scripts': [
            'azarashi=azarashi.__main__:main',
        ],
    },
)



from copy import deepcopy
from datetime import datetime, timedelta

from pprint import pformat
from ..exception import QzssDcrDecoderException


class QzssDcReportBase:
    def __init__(self,
                 sentence,
                 raw=None,
                 timestamp=None,
                 **kwargs):
        self.sentence = sentence
        if raw is None:
            self.raw = b''
        if timestamp is None:
            timestamp = datetime.now()
        self.timestamp = timestamp

    def __eq__(self, other):
        if type(self) != type(other):
            return False
        return self.raw == other.raw

    def __str__(self):
        str(self.__dict__)

    def get_params(self):
        return deepcopy(self.__dict__)


class QzssDcReportMessagePartial(QzssDcReportBase):
    def __init__(self,
                 message,
                 nmea,
                 message_header=None,
                 satellite_id=None,
                 satellite_prn=None,
                 sentence=None,
                 **kwargs):
        if sentence is None:
            sentence = message
        super().__init__(sentence, **kwargs)
        self.message = message
        self.nmea = nmea
        self.message_header = message_header
        self.satellite_id = satellite_id
        self.satellite_prn = satellite_prn
        if self.message[1] >> 2 == 44: # DCX
            # starts from camf, discards pab, mt and sd fields.
            self.raw = self.message[3:27] + bytes((self.message[27] & 0xF0,))
        else:
            self.raw = self.message[1:27] + bytes((self.message[27] & 0xF0,))

class QzssDcReportMessageBase(QzssDcReportMessagePartial):
    def __init__(self,
                 preamble,
                 message_type,
                 **kwargs):
        super().__init__(**kwargs)
        self.preamble = preamble
        self.message_type = message_type


class QzssDcReportJmaBase(QzssDcReportMessageBase):
    def __init__(self,
                 version,
                 report_classification,
                 report_classification_en,
                 report_classification_no,
                 disaster_category,
                 disaster_category_en,
                 disaster_category_no,
                 report_time,
                 information_type,
                 information_type_en,
                 information_type_no,
                 **kwargs):
        super().__init__(**kwargs)
        self.version = version
        self.report_classification = report_classification
        self.report_classification_en = report_classification_en
        self.report_classification_no = report_classification_no
        self.disaster_category = disaster_category
        self.disaster_category_en = disaster_category_en
        self.disaster_category_no = disaster_category_no
        self.report_time = report_time
        self.information_type = information_type
        self.information_type_en = information_type_en
        self.information_type_no = information_type_no

    def get_header(self):
        header = f'防災気象情報({self.disaster_category})' + \
                 f'({self.information_type})' + \
                 f'({self.report_classification})'
        if self.report_classification_no == 7:
            header += '\n*** これは訓練です ***'
        elif self.information_type_no == 2:
            header += '\n*** 取り消しされました ***'
        return header

    def get_header_en(self):
        header = f'JMA-DC Report ({self.disaster_category_en})' + \
                 f' ({self.information_type_en})' + \
                 f' ({self.report_classification_en})'
        if self.report_classification_no == 7:
            header += '\n*** TRAINING/TEST ***'
        elif self.information_type_no == 2:
            header += '\n*** CANCELLATION ***'
        return header

    def get_report_time_str(self, time_diff=9):
        at = self.report_time + timedelta(hours=time_diff)
        return f'{at.month}月{at.day}日{at.hour}時{at.minute}分'

    def get_report_time_str_iso(self):
        return self.report_time.strftime('--%m-%dT%H:%MZ')

    @staticmethod
    def convert_dt_to_str(dt, time_diff=9):
        dt += timedelta(hours=time_diff)
        return f'{dt.day}日{dt.hour}時{dt.minute}分'

    @staticmethod
    def convert_dt_to_str_iso(dt):
        return dt.strftime('---%dT%H:%MZ')

    def convert_dt_to_ambiguous_time_str(self, td, du, time_diff=9):
        td += timedelta(hours=time_diff)
        try:
            return [f'{td.month}月{td.day}日{td.hour}時{td.minute}分',  # No ambiguity
                    f'{td.month}月{td.day}日{td.hour}時{td.minute}分頃',
                    # Approximate time(equivalent to Approximate time (minute))
                    f'{td.month}月{td.day}日{td.hour}時{td.minute}分',  # Approximate time(second)
                    f'{td.month}月{td.day}日{td.hour}時{td.minute}分頃',  # Approximate time(minute)
                    f'{td.month}月{td.day}日{td.hour}時頃',  # Approximate time(hour)
                    f'{td.month}月{td.day}日頃',  # Approximate time(day)
                    f'{td.month}月頃',  # Approximate time(month)
                    f'{td.year}年頃',  # Approximate time(year)
                    ][du]
        except:
            raise QzssDcrDecoderException(
                f'Undefined JMA Ambiguity of Activity Time: {du}',
                self)

    @staticmethod
    def convert_lat_lon_to_str(coordinates):
        return f'{"北緯" if coordinates["lat_ns"] == 0 else "南緯"}' + \
            f'{coordinates["lat_d"]}度' + \
            f'{coordinates["lat_m"]}分' + \
            f'{coordinates["lat_s"]}秒 ' + \
            f'{"東経" if coordinates["lon_ew"] == 0 else "西経"}' + \
            f'{coordinates["lon_d"]}度' + \
            f'{coordinates["lon_m"]}分' + \
            f'{coordinates["lon_s"]}秒'


class QzssDcReportJmaEarthquakeEarlyWarning(QzssDcReportJmaBase):
    def __init__(self,
                 long_period_ground_motion_lower_limit,
                 long_period_ground_motion_lower_limit_raw,
                 long_period_ground_motion_upper_limit,
                 long_period_ground_motion_upper_limit_raw,
                 notifications_on_disaster_prevention,
                 notifications_on_disaster_prevention_raw,
                 occurrence_time_of_earthquake,
                 depth_of_hypocenter,
                 depth_of_hypocenter_raw,
                 magnitude,
                 magnitude_raw,
                 assumptive,
                 seismic_epicenter,
                 seismic_epicenter_raw,
                 seismic_intensity_lower_limit,
                 seismic_intensity_lower_limit_raw,
                 seismic_intensity_upper_limit,
                 seismic_intensity_upper_limit_raw,
                 eew_forecast_regions,
                 eew_forecast_regions_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.long_period_ground_motion_lower_limit = long_period_ground_motion_lower_limit
        self.long_period_ground_motion_lower_limit_raw = long_period_ground_motion_lower_limit_raw
        self.long_period_ground_motion_upper_limit = long_period_ground_motion_upper_limit
        self.long_period_ground_motion_upper_limit_raw = long_period_ground_motion_upper_limit_raw
        self.notifications_on_disaster_prevention = notifications_on_disaster_prevention
        self.notifications_on_disaster_prevention_raw = notifications_on_disaster_prevention_raw
        self.occurrence_time_of_earthquake = occurrence_time_of_earthquake
        self.depth_of_hypocenter = depth_of_hypocenter
        self.depth_of_hypocenter_raw = depth_of_hypocenter_raw
        self.magnitude = magnitude
        self.magnitude_raw = magnitude_raw
        self.assumptive = assumptive
        self.seismic_epicenter = seismic_epicenter
        self.seismic_epicenter_raw = seismic_epicenter_raw
        self.seismic_intensity_lower_limit = seismic_intensity_lower_limit
        self.seismic_intensity_lower_limit_raw = seismic_intensity_lower_limit_raw
        self.seismic_intensity_upper_limit = seismic_intensity_upper_limit
        self.seismic_intensity_upper_limit_raw = seismic_intensity_upper_limit_raw
        self.eew_forecast_regions = eew_forecast_regions
        self.eew_forecast_regions_raw = eew_forecast_regions_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 '緊急地震速報\n'

        report += '\n'.join(self.notifications_on_disaster_prevention)

        if self.assumptive is True:
            assumptive_str = '(仮定震源要素)'
        else:
            assumptive_str = ''

        report += f'\n\n発表時刻: {self.get_report_time_str()}\n\n' + \
                  f'震央地名: {self.seismic_epicenter}\n' + \
                  f'地震発生時刻: {self.convert_dt_to_str(self.occurrence_time_of_earthquake)}\n' + \
                  f'深さ: {self.depth_of_hypocenter}{assumptive_str}\n' + \
                  f'マグニチュード: {self.magnitude}{assumptive_str}\n' + \
                  f'震度(下限): {self.seismic_intensity_lower_limit}\n' + \
                  f'震度(上限): {self.seismic_intensity_upper_limit}\n'
        if self.long_period_ground_motion_lower_limit != None:
            report += f'長周期地震動階級(下限): {self.long_period_ground_motion_lower_limit}\n'
        if self.long_period_ground_motion_upper_limit != None:
            report += f'長周期地震動階級(上限): {self.long_period_ground_motion_upper_limit}\n'
        report += '、'.join(self.eew_forecast_regions)
        return report


class QzssDcReportJmaHypocenter(QzssDcReportJmaBase):
    def __init__(self,
                 notifications_on_disaster_prevention,
                 notifications_on_disaster_prevention_raw,
                 occurrence_time_of_earthquake,
                 depth_of_hypocenter,
                 depth_of_hypocenter_raw,
                 magnitude,
                 magnitude_raw,
                 seismic_epicenter,
                 seismic_epicenter_raw,
                 coordinates_of_hypocenter,
                 **kwargs):
        super().__init__(**kwargs)
        self.notifications_on_disaster_prevention = notifications_on_disaster_prevention
        self.notifications_on_disaster_prevention_raw = notifications_on_disaster_prevention_raw
        self.occurrence_time_of_earthquake = occurrence_time_of_earthquake
        self.depth_of_hypocenter = depth_of_hypocenter
        self.depth_of_hypocenter_raw = depth_of_hypocenter_raw
        self.magnitude = magnitude
        self.magnitude_raw = magnitude_raw
        self.seismic_epicenter = seismic_epicenter
        self.seismic_epicenter_raw = seismic_epicenter_raw
        self.coordinates_of_hypocenter = coordinates_of_hypocenter

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 f'{self.convert_dt_to_str(self.occurrence_time_of_earthquake)}' + \
                 'ころ、地震がありました。\n'

        report += '\n'.join(self.notifications_on_disaster_prevention)

        report += f'\n\n発表時刻: {self.get_report_time_str()}\n\n' + \
                  f'震央地名: {self.seismic_epicenter}\n' + \
                  f'緯度・経度: {self.convert_lat_lon_to_str(self.coordinates_of_hypocenter)}\n' + \
                  f'深さ: {self.depth_of_hypocenter}\n' + \
                  f'マグニチュード: {self.magnitude}'
        return report


class QzssDcReportJmaSeismicIntensity(QzssDcReportJmaBase):
    def __init__(self,
                 occurrence_time_of_earthquake,
                 seismic_intensities,
                 seismic_intensities_raw,
                 prefectures,
                 prefectures_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.occurrence_time_of_earthquake = occurrence_time_of_earthquake
        self.seismic_intensities = seismic_intensities
        self.seismic_intensities_raw = seismic_intensities_raw
        self.prefectures = prefectures
        self.prefectures_raw = prefectures_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 f'{self.convert_dt_to_str(self.occurrence_time_of_earthquake)}' + \
                 'ころ、地震による強い揺れを感じました。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}'

        for i in range(len(self.seismic_intensities)):
            report += f'\n\n震度: {self.seismic_intensities[i]}\n' + \
                      f'{self.prefectures[i]}'
        return report


class QzssDcReportJmaNankaiTroughEarthquake(QzssDcReportJmaBase):
    completed = False
    reports = {}

    def __init__(self,
                 information_serial_code,
                 information_serial_code_raw,
                 text_information,
                 page_number,
                 total_page,
                 **kwargs):
        super().__init__(**kwargs)
        self.information_serial_code = information_serial_code
        self.information_serial_code_raw = information_serial_code_raw
        self.text_information = text_information
        self.page_number = page_number
        self.total_page = total_page

        cls = self.__class__
        ex_report = cls.reports.get(self.page_number)
        if ex_report is not None:
            if ex_report == self:
                return
            else:
                cls.completed = False
                cls.reports = {}

        cls.reports.update({self.page_number: self})
        if len(cls.reports) == self.total_page:
            cls.completed = True

    def extract_text_information(self):
        cls = self.__class__
        if cls.completed is not True:
            return f'受信中 ({self.page_number}) [{len(cls.reports)}/{self.total_page}]'

        cls = self.__class__
        msg_bytes = b''
        for i in range(1, self.total_page + 1):
            msg_bytes += cls.reports[i].text_information

        return msg_bytes.decode('utf-8', errors='ignore')

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 f'南海トラフ地震に関連する情報が発表されました。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}\n' + \
                 f'地震関連情報: {self.information_serial_code}\n' + \
                 f'{self.extract_text_information()}'
        return report


class QzssDcReportJmaTsunami(QzssDcReportJmaBase):
    def __init__(self,
                 notifications_on_disaster_prevention,
                 notifications_on_disaster_prevention_raw,
                 tsunami_warning_code,
                 tsunami_warning_code_raw,
                 expected_tsunami_arrival_times,
                 tsunami_heights,
                 tsunami_heights_raw,
                 tsunami_forecast_regions,
                 tsunami_forecast_regions_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.notifications_on_disaster_prevention = notifications_on_disaster_prevention
        self.notifications_on_disaster_prevention_raw = notifications_on_disaster_prevention_raw
        self.tsunami_warning_code = tsunami_warning_code
        self.tsunami_warning_code_raw = tsunami_warning_code_raw
        self.expected_tsunami_arrival_times = expected_tsunami_arrival_times
        self.tsunami_heights = tsunami_heights
        self.tsunami_heights_raw = tsunami_heights_raw
        self.tsunami_forecast_regions = tsunami_forecast_regions
        self.tsunami_forecast_regions_raw = tsunami_forecast_regions_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 f'{self.tsunami_warning_code}を発表しました。\n'

        report += '\n'.join(self.notifications_on_disaster_prevention)

        report += f'\n\n発表時刻: {self.get_report_time_str()}'

        for i in range(len(self.expected_tsunami_arrival_times)):
            if self.expected_tsunami_arrival_times[i] is None:
                ta = '不明'
            else:
                ta = self.convert_dt_to_str(self.expected_tsunami_arrival_times[i])
            report += f'\n\n津波到達予想時刻: {ta}\n' + \
                      f'津波の高さ: {self.tsunami_heights[i]}\n' + \
                      f'{self.tsunami_forecast_regions[i]}'
        return report


class QzssDcReportJmaNorthwestPacificTsunami(QzssDcReportJmaBase):
    def __init__(self,
                 tsunamigenic_potential_en,
                 tsunamigenic_potential_raw,
                 expected_tsunami_arrival_times,
                 tsunami_heights_en,
                 tsunami_heights_raw,
                 coastal_regions_en,
                 coastal_regions_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.tsunamigenic_potential_en = tsunamigenic_potential_en
        self.tsunamigenic_potential_raw = tsunamigenic_potential_raw
        self.expected_tsunami_arrival_times = expected_tsunami_arrival_times
        self.tsunami_heights_en = tsunami_heights_en
        self.tsunami_heights_raw = tsunami_heights_raw
        self.coastal_regions_en = coastal_regions_en
        self.coastal_regions_raw = coastal_regions_raw

    def __str__(self):
        report = f'{self.get_header_en()}\n' + \
                 f'{self.tsunamigenic_potential_en}.\n\n' + \
                 f'Time of Issue: {self.get_report_time_str_iso()}'

        for i in range(len(self.expected_tsunami_arrival_times)):
            if self.expected_tsunami_arrival_times[i] is None:
                ta = 'Unknown'
            else:
                ta = self.convert_dt_to_str_iso(self.expected_tsunami_arrival_times[i])
            report += f'\n\nExpected Tsunami Arrival Time: {ta}\n' + \
                      f'Tsunami Height: {self.tsunami_heights_en[i]}\n' + \
                      f'Coastal Region: {self.coastal_regions_en[i]}'
        return report


class QzssDcReportJmaVolcano(QzssDcReportJmaBase):
    def __init__(self,
                 ambiguity_of_activity_time_no,
                 activity_time,
                 volcanic_warning_code,
                 volcanic_warning_code_raw,
                 volcano_name,
                 volcano_name_raw,
                 local_governments,
                 local_governments_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.ambiguity_of_activity_time_no = ambiguity_of_activity_time_no
        self.activity_time = activity_time
        self.volcanic_warning_code = volcanic_warning_code
        self.volcanic_warning_code_raw = volcanic_warning_code_raw
        self.volcano_name = volcano_name
        self.volcano_name_raw = volcano_name_raw
        self.local_governments = local_governments
        self.local_governments_raw = local_governments_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 '火山に関連する情報をお知らせします。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}\n\n' + \
                 f'火山名: {self.volcano_name}\n' + \
                 f'日時: {self.convert_dt_to_ambiguous_time_str(self.activity_time, self.ambiguity_of_activity_time_no)}\n' + \
                 f'現象: {self.volcanic_warning_code}\n\n'

        report += '、'.join(self.local_governments)
        return report


class QzssDcReportJmaAshFall(QzssDcReportJmaBase):
    def __init__(self,
                 activity_time,
                 ash_fall_warning_type,
                 ash_fall_warning_type_raw,
                 volcano_name,
                 volcano_name_raw,
                 expected_ash_fall_times,
                 ash_fall_warning_codes,
                 ash_fall_warning_codes_raw,
                 local_governments,
                 local_governments_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.activity_time = activity_time
        self.ash_fall_warning_type = ash_fall_warning_type
        self.ash_fall_warning_type_raw = ash_fall_warning_type_raw
        self.volcano_name = volcano_name
        self.volcano_name_raw = volcano_name_raw
        self.expected_ash_fall_times = expected_ash_fall_times
        self.ash_fall_warning_codes = ash_fall_warning_codes
        self.ash_fall_warning_codes_raw = ash_fall_warning_codes_raw
        self.local_governments = local_governments
        self.local_governments_raw = local_governments_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 '降灰に関連する情報をお知らせします。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}\n\n' + \
                 f'{self.ash_fall_warning_type}\n' + \
                 f'火山名: {self.volcano_name}\n' + \
                 f'日時: {self.convert_dt_to_str(self.activity_time)}'

        for i in range(len(self.expected_ash_fall_times)):
            report += '\n\n' + \
                      f'基点時刻からの時間: {self.expected_ash_fall_times[i]}時間\n' + \
                      f'現象: {self.ash_fall_warning_codes[i]}\n' + \
                      f'{self.local_governments[i]}'
        return report


class QzssDcReportJmaWeather(QzssDcReportJmaBase):
    def __init__(self,
                 weather_warning_state,
                 weather_warning_state_raw,
                 weather_related_disaster_sub_categories,
                 weather_related_disaster_sub_categories_raw,
                 weather_forecast_regions,
                 weather_forecast_regions_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.weather_warning_state = weather_warning_state
        self.weather_warning_state_raw = weather_warning_state_raw
        self.weather_related_disaster_sub_categories = weather_related_disaster_sub_categories
        self.weather_related_disaster_sub_categories_raw = weather_related_disaster_sub_categories_raw
        self.weather_forecast_regions = weather_forecast_regions
        self.weather_forecast_regions_raw = weather_forecast_regions_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 '気象に関連する情報をお知らせします。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}'

        for i in range(len(self.weather_related_disaster_sub_categories)):
            report += f'\n\n警報等情報要素: {self.weather_related_disaster_sub_categories[i]}' + \
                      f'({self.weather_warning_state})\n' + \
                      f'{self.weather_forecast_regions[i]}'
        return report


class QzssDcReportJmaFlood(QzssDcReportJmaBase):
    def __init__(self,
                 flood_warning_levels,
                 flood_warning_levels_raw,
                 flood_forecast_regions,
                 flood_forecast_regions_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.flood_warning_levels = flood_warning_levels
        self.flood_warning_levels_raw = flood_warning_levels_raw
        self.flood_forecast_regions = flood_forecast_regions
        self.flood_forecast_regions_raw = flood_forecast_regions_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 '河川の氾濫に関連する情報をお知らせします。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}'

        for i in range(len(self.flood_warning_levels)):
            report += f'\n\n警報レベル: {self.flood_warning_levels[i]}\n' + \
                      f'{self.flood_forecast_regions[i]}'
        return report


class QzssDcReportJmaMarine(QzssDcReportJmaBase):
    def __init__(self,
                 marine_warning_codes,
                 marine_warning_codes_raw,
                 marine_forecast_regions,
                 marine_forecast_regions_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.marine_warning_codes = marine_warning_codes
        self.marine_warning_codes_raw = marine_warning_codes_raw
        self.marine_forecast_regions = marine_forecast_regions
        self.marine_forecast_regions_raw = marine_forecast_regions_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 '海上警報が発表されました。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}'

        for i in range(len(self.marine_warning_codes)):
            report += f'\n\n警報等情報要素: {self.marine_warning_codes[i]}\n' + \
                      f'{self.marine_forecast_regions[i]}'
        return report


class QzssDcReportJmaTyphoon(QzssDcReportJmaBase):
    def __init__(self,
                 reference_time,
                 reference_time_type,
                 reference_time_type_raw,
                 elapsed_time_from_reference_time,
                 typhoon_number,
                 typhoon_number_raw,
                 typhoon_scale_category,
                 typhoon_scale_category_raw,
                 typhoon_intensity_category,
                 typhoon_intensity_category_raw,
                 coordinates_of_typhoon,
                 central_pressure,
                 central_pressure_raw,
                 maximum_wind_speed,
                 maximum_wind_speed_raw,
                 maximum_gust_wind_speed,
                 maximum_gust_wind_speed_raw,
                 **kwargs):
        super().__init__(**kwargs)
        self.reference_time = reference_time
        self.reference_time_type = reference_time_type
        self.reference_time_type_raw = reference_time_type_raw
        # the time elapsed from the analysis
        self.elapsed_time_from_reference_time = elapsed_time_from_reference_time
        self.typhoon_number = typhoon_number
        self.typhoon_number_raw = typhoon_number_raw
        self.typhoon_scale_category = typhoon_scale_category
        self.typhoon_scale_category_raw = typhoon_scale_category_raw
        self.typhoon_intensity_category = typhoon_intensity_category
        self.typhoon_intensity_category_raw = typhoon_intensity_category_raw
        self.coordinates_of_typhoon = coordinates_of_typhoon
        self.central_pressure = central_pressure
        self.central_pressure_raw = central_pressure_raw
        self.maximum_wind_speed = maximum_wind_speed
        self.maximum_wind_speed_raw = maximum_wind_speed_raw
        self.maximum_gust_wind_speed = maximum_gust_wind_speed
        self.maximum_gust_wind_speed_raw = maximum_gust_wind_speed_raw

    def __str__(self):
        report = f'{self.get_header()}\n' + \
                 '台風解析・予報情報が発表されました。\n\n' + \
                 f'発表時刻: {self.get_report_time_str()}\n\n' + \
                 f'台風番号: {self.typhoon_number}\n' + \
                 f'基点時刻: {self.convert_dt_to_str(self.reference_time)}\n' + \
                 f'基点時刻分類: {self.reference_time_type}\n' + \
                 f'情報の基点時刻(実況)からの経過時間: {self.elapsed_time_from_reference_time}時間後\n' + \
                 f'大きさ: {self.typhoon_scale_category}\n' + \
                 f'強さ: {self.typhoon_intensity_category}\n' + \
                 f'緯度・経度: {self.convert_lat_lon_to_str(self.coordinates_of_typhoon)}\n' + \
                 f'中心気圧: {self.central_pressure}\n' + \
                 f'最大風速: {self.maximum_wind_speed}\n' + \
                 f'最大瞬間風速: {self.maximum_gust_wind_speed}'
        return report


class QzssDcXtendedMessageBase(QzssDcReportMessagePartial):
    def __init__(self,
                 preamble,
                 message_type,
                 camf,
                 ignore_a12_to_a16,
                 ignore_a17_to_a18,
                 ignore_ex1,
                 ignore_ex2_to_ex7,
                 ignore_ex8_to_ex9,
                 **kwargs):
        super().__init__(**kwargs)
        self.preamble = preamble
        self.message_type = message_type
        self.camf = camf
        self.ignore_a12_to_a16 = ignore_a12_to_a16
        self.ignore_a17_to_a18 = ignore_a17_to_a18
        self.ignore_ex1 = ignore_ex1
        self.ignore_ex2_to_ex7 = ignore_ex2_to_ex7
        self.ignore_ex8_to_ex9 = ignore_ex8_to_ex9

        for key, value in kwargs.items():
            if key not in self.__dict__:
                self.__dict__[key] = value

    def __str__(self):
        header = f"### DCX Message - {self.__dict__.get('dcx_message_type')} ###\n"
        if self.camf.a1 == 0:
                header += "*** This is a test message ***\n"

        report = header + \
                 f"A1 - Message type: {self.__dict__.get('a1_message_type')}\n" + \
                 f"A2 - Country/region name: {self.__dict__.get('a2_country_region_name')}\n" + \
                 f"A3 - Provider identifier: {self.__dict__.get('a3_provider_identifier')}\n" + \
                 f"A4 - Hazard category and type: {self.__dict__.get('a4_hazard_category')} - " + \
                 f"{self.__dict__.get('a4_hazard_type')}\n" + \
                 f"A4 - Hazard definition: {self.__dict__.get('a4_hazard_definition')}\n" + \
                 f"A5 - Severity: {self.__dict__.get('a5_severity')}\n" + \
                 f"A6A7 - Hazard onset: {self.__dict__.get('a6a7_hazard_onset_datetime')}\n" + \
                 f"A8 - Hazard duration: {self.__dict__.get('a8_hazard_duration')}\n"
        if self.camf.a9 == 0: # international
            report += f"A11 - Guidance to react code: {self.__dict__.get('a11_international_library_code')}\n"
            if self.camf.a11 != 0:
                report += f"A11 - Guidance to react: {self.__dict__.get('a11_international_library')}\n"
        elif self.camf.a9 == 1: # japanese
            if self.camf.a11 != 0:
                report += f"A11 - Guidance to react: {self.__dict__.get('a11_japanese_library')}\n" + \
                          f"A11 - Guidance to react (ja): {self.__dict__.get('a11_japanese_library_ja')}\n"

        if self.ignore_a12_to_a16 is False:
            report += f"A12 - Ellipse centre latitude: {round(self.__dict__.get('a12_ellipse_centre_latitude'), 3)}\n" + \
                      f"A13 - Ellipse centre longitude: {round(self.__dict__.get('a13_ellipse_centre_longitude'), 3)}\n" + \
                      f"A14 - Ellipse semi - major axis: {round(self.__dict__.get('a14_ellipse_semi_major_axis'), 3)}\n" + \
                      f"A15 - Ellipse semi - minor axis: {round(self.__dict__.get('a15_ellipse_semi_minor_axis'), 3)}\n" + \
                      f"A16 - Ellipse azimuth: {round(self.__dict__.get('a16_ellipse_azimuth'), 3)}\n"

        if self.ignore_a17_to_a18 is False:
            a17 =  self.__dict__.get('a17_main_subject_for_specific_settings')
            if a17 is not None:
                report += f"A17 - Specific settings: {a17}\n"
                keys = self.__dict__.keys()
                prefix = ['c%d_' % (i + 1) for i in range(10)] + ['d%d_' % (i + 1) for i in range(34)]
                for k in keys:
                    for p in prefix:
                        if k.startswith(p):
                            ident, title = k.split('_', 1)
                            headline = ident.upper() + ' - ' + title.replace('_', ' ').capitalize()
                            content = self.__dict__.get(k)
                            if type(content) is float:
                                content = round(content, 3)
                            report += f"{headline}: {content}\n"

        if self.ignore_ex1 is False:
            report += f"EX1 - Target area: {self.__dict__.get('ex1_target_area')}\n" + \
                      f"EX1 - Target area (ja): {self.__dict__.get('ex1_target_area_ja')}\n"

        if self.ignore_ex2_to_ex7 is False:
            report += f"EX2 - Evacuate direction type: {self.__dict__.get('ex2_evacuate_direction_type')}\n" + \
                      f"EX3 - Additional ellipse centre latitude: {round(self.__dict__.get('ex3_additional_ellipse_centre_latitude'), 3)}\n" + \
                      f"EX4 - Additional ellipse centre longitude: {round(self.__dict__.get('ex4_additional_ellipse_centre_longitude'), 3)}\n" + \
                      f"EX5 - Additional ellipse semi major axis: {round(self.__dict__.get('ex5_additional_ellipse_semi_major_axis'), 3)}\n" + \
                      f"EX6 - Additional ellipse semi minor axis: {round(self.__dict__.get('ex6_additional_ellipse_semi_minor_axis'), 3)}\n" + \
                      f"EX7 - Additional ellipse azimuth: {round(self.__dict__.get('ex7_additional_ellipse_azimuth'), 3)}\n"

        if self.ignore_ex8_to_ex9 is False:
            report += f"EX8 - Target area list type: {self.__dict__.get('ex8_target_area_list_type')}\n" + \
                      f"EX9 - Target area list: {self.__dict__.get('ex9_target_area_list')}\n" + \
                      f"EX9 - Target area list (ja): {self.__dict__.get('ex9_target_area_list_ja')}\n"

        if report.endswith('\n'):
            report = report[:-1]

        return report


class QzssDcxNullMsg(QzssDcXtendedMessageBase):
    def __init__(self,
                 **kwargs):
        super().__init__(**kwargs)

    def __str__(self):
        return f"### DCX Message - {self.__dict__.get('dcx_message_type')} ###"


class QzssDcxOutsideJapan(QzssDcXtendedMessageBase):
    def __init__(self,
                 **kwargs):
        super().__init__(**kwargs)


class QzssDcxLAlert(QzssDcXtendedMessageBase):
    def __init__(self,
                 **kwargs):
        super().__init__(**kwargs)


class QzssDcxJAlert(QzssDcXtendedMessageBase):
    def __init__(self,
                 **kwargs):
        super().__init__(**kwargs)


class QzssDcxMTInfo(QzssDcXtendedMessageBase):
    def __init__(self,
                 **kwargs):
        super().__init__(**kwargs)


class QzssDcxUnknown(QzssDcXtendedMessageBase):
    def __init__(self,
                 **kwargs):
        super().__init__(**kwargs)



from .qzss_dcr_decoder import QzssDcrDecoder
from .qzss_dcr_decoder_base import QzssDcrDecoderBase
from ..exception import QzssDcrDecoderException
from ..report import QzssDcReportBase


class HexQzssDcrDecoder(QzssDcrDecoderBase):
    schema = QzssDcReportBase

    def decode(self):
        if not self.sentence:
            raise EOFError('Encountered EOF')

        self.sentence = self.sentence.strip()

        if len(self.sentence) < 63:
            raise QzssDcrDecoderException(
                'Too Short Sentence',
                self)
        if len(self.sentence) > 63:
            raise QzssDcrDecoderException(
                'Too Long Sentence',
                self)

        # converts the message to bytes type
        try:
            self.message = bytes.fromhex(self.sentence + '0')
        except ValueError:
            raise QzssDcrDecoderException(
                'Invalid Message',
                self)

        self.nmea = self.message_to_nmea()

        # stacks the next decoder
        return QzssDcrDecoder(**self.get_params()).decode()



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_jma_disaster_category = QzssDcrDefinition(
    {
        1: '緊急地震速報',
        2: '震源',
        3: '震度',
        4: '南海トラフ地震',
        5: '津波',
        6: '北西太平洋津波',
        # 7 : '未使用',
        8: '火山',
        9: '降灰',
        10: '気象',
        11: '洪水',
        12: '台風',
        # 13: '未使用',
        14: '海上',
    },
    undefined=None
)

qzss_dcr_jma_disaster_category_en = QzssDcrDefinition(
    {
        1: 'Earthquake Early Warning',
        2: 'Hypocenter',
        3: 'Seismic Intensity',
        4: 'Nankai Trough Earthquake',
        5: 'Tsunami',
        6: 'Northwest Pacific Tsunami',
        # 7 : 'Unused',
        8: 'Volcano',
        9: 'Ash Fall',
        10: 'Weather',
        11: 'Flood',
        12: 'Typhoon',
        # 13: 'Unused',
        14: 'Marine',
    },
    undefined=None
)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_jma_ash_fall_warning_code = QzssDcrDefinition(
    {
        1: "少量の降灰",
        2: "やや多量の降灰",
        3: "多量の降灰",
        4: "小さな噴石の落下",
        7: "その他の防災気象情報要素2",
        # "N*": "防災気象情報要素2(コード番号：N)",
    },
    undefined="防災気象情報要素2(コード番号：%d)"
)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_jma_marine_warning_code = QzssDcrDefinition(
    {
        0: "海上警報解除",
        10: "海上着氷警報",
        11: "海上濃霧警報",
        12: "海上うねり警報",
        20: "海上風警報",
        21: "海上強風警報",
        22: "海上暴風警報",
        23: "海上台風警報",
        31: "その他の警報等情報要素 海上警報",
        # 'NN*': "警報等情報要素_海上警報(コード番号：NN)",
    },
    undefined="警報等情報要素_海上警報(コード番号：%d)"
)



from .nmea_qzss_dcr_message_header import *
from .qzss_dcr_jma_ash_fall_warning_code import *
from .qzss_dcr_jma_coastal_region import *
from .qzss_dcr_jma_disaster_category import *
from .qzss_dcr_jma_eew_forecast_region import *
from .qzss_dcr_jma_epicenter_and_hypocenter import *
from .qzss_dcr_jma_flood_forecast_region import *
from .qzss_dcr_jma_flood_warning_level import *
from .qzss_dcr_jma_information_serial_code import *
from .qzss_dcr_jma_information_type import *
from .qzss_dcr_jma_local_government import *
from .qzss_dcr_jma_long_period_ground_motion_lower_limit import *
from .qzss_dcr_jma_long_period_ground_motion_upper_limit import *
from .qzss_dcr_jma_marine_forecast_region import *
from .qzss_dcr_jma_marine_warning_code import *
from .qzss_dcr_jma_northwest_pacific_tsunami_height import *
from .qzss_dcr_jma_notification_on_disaster_prevention import *
from .qzss_dcr_jma_prefecture import *
from .qzss_dcr_jma_seismic_intensity import *
from .qzss_dcr_jma_seismic_intensity_lower_limit import *
from .qzss_dcr_jma_seismic_intensity_upper_limit import *
from .qzss_dcr_jma_tsunami_forecast_region import *
from .qzss_dcr_jma_tsunami_height import *
from .qzss_dcr_jma_tsunami_warning_code import *
from .qzss_dcr_jma_tsunamigenic_potential import *
from .qzss_dcr_jma_typhoon_intensity_category import *
from .qzss_dcr_jma_typhoon_reference_time_type import *
from .qzss_dcr_jma_typhoon_scale_category import *
from .qzss_dcr_jma_volcanic_warning_code import *
from .qzss_dcr_jma_volcano_name import *
from .qzss_dcr_jma_weather_forecast_region import *
from .qzss_dcr_jma_weather_related_disaster_sub_category import *
from .qzss_dcr_jma_weather_warning_state import *
from .qzss_dcr_message_type import *
from .qzss_dcr_preamble import *
from .qzss_dcr_jma_report_classification import *
from .qzss_dcr_satellite_prn import *
from .qzss_dcx_message_type import *
from .qzss_dcx_camf_a1_message_type import *
from .qzss_dcx_camf_a2_country_region_name import *
from .qzss_dcx_camf_a3_provider_identifier import *
from .qzss_dcx_camf_a4_hazard_category_and_type import *
from .qzss_dcx_camf_a5_severity import *
from .qzss_dcx_camf_a6_hazard_onset_week import *
from .qzss_dcx_camf_a8_hazard_duration import *
from .qzss_dcx_camf_a9_selection_of_library import *
from .qzss_dcx_camf_a10_library_version import *
from .qzss_dcx_camf_a11_international_library import *
from .qzss_dcx_camf_a11_japanese_library import *
from .qzss_dcx_camf_a17_main_subject_for_specific_settings import *
from .qzss_dcx_camf_b4_lower_level_fields_tables import *
from .qzss_dcx_camf_c10_guicance_library_for_second_ellipse import *
from .qzss_dcx_camf_ex1_target_area_code import *
from .qzss_dcx_camf_ex9_target_area_code import *
from .ublox_qzss_dcr_message_header import *
from .ublox_qzss_svid_prn_map import *



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_jma_notification_on_disaster_prevention = QzssDcrDefinition(
    {
        0: "なし",
        101: "今後若干の海面変動があるかもしれません。",
        102: "今後若干の海面変動があるかもしれませんが、被害の心配はありません。",
        103: "今後もしばらく海面変動が続くと思われます。",
        104: "今後もしばらく海面変動が続くと思われますので、海水浴や磯釣り等を行う際は注意してください。",
        105: "今後もしばらく海面変動が続くと思われますので、磯釣り等を行う際は注意してください。",
        107: "現在、大津波警報・津波警報・津波注意報を発表している沿岸はありません。",
        109: "津波と満潮が重なると、津波はより高くなりますので一層厳重な警戒が必要です。",
        110: "津波と満潮が重なると、津波はより高くなりますので十分な注意が必要です。",
        111: "場所によっては、観測した津波の高さよりさらに大きな津波が到達しているおそれがあります。",
        112: "今後、津波の高さは更に高くなることも考えられます。",
        113: "沖合での観測値をもとに津波が推定されている沿岸では、早いところでは、既に津波が到達していると推定されます。",
        114: "津波による潮位変化が観測されてから最大波が観測されるまでに数時間以上かかることがあります。",
        115: "沖合での観測値であり、沿岸では津波はさらに高くなります。",
        121: "＜大津波警報＞\n" + \
             "大きな津波が襲い甚大な被害が発生します。\n" + \
             "沿岸部や川沿いにいる人はただちに高台や避難ビルなど安全な場所へ避難してください。\n" + \
             "津波は繰り返し襲ってきます。警報が解除されるまで安全な場所から離れないでください。",
        122: "＜津波警報＞\n" + \
             "津波による被害が発生します。\n" + \
             "沿岸部や川沿いにいる人はただちに高台や避難ビルなど安全な場所へ避難してください。\n" + \
             "津波は繰り返し襲ってきます。警報が解除されるまで安全な場所から離れないでください。",
        123: "＜津波注意報＞\n" + \
             "海の中や海岸付近は危険です。\n" + \
             "海の中にいる人はただちに海から上がって、海岸から離れてください。\n" + \
             "潮の流れが速い状態が続きますので、注意報が解除されるまで海に入ったり海岸に近づいたりしないようにしてください。",
        124: "＜津波予報(若干の海面変動)＞\n" + \
             "若干の海面変動が予想されますが、被害の心配はありません。",
        131: "警報が発表された沿岸部や川沿いにいる人はただちに高台や避難ビルなど安全な場所へ避難してください。\n" + \
             "到達予想時刻は、予報区のなかで最も早く津波が到達する時刻です。場所によっては、この時刻よりもかなり遅れて津波が襲ってくることがあります。\n" + \
             "到達予想時刻から津波が最も高くなるまでに数時間以上かかることがありますので、観測された津波の高さにかかわらず、警報が解除されるまで安全な場所から離れないでください。",
        132: "場所によっては津波の高さが「予想される津波の高さ」より高くなる可能性があります。",
        141: "東日本大震災クラスの津波が来襲します。",
        142: "沖合で高い津波を観測したため大津波警報・津波警報に切り替えました。",
        143: "沖合で高い津波を観測したため大津波警報・津波警報を切り替えました。",
        144: "沖合で高い津波を観測したため大津波警報に切り替えました。",
        145: "沖合で高い津波を観測したため大津波警報を切り替えました。",
        146: "沖合で高い津波を観測したため津波警報に切り替えました。",
        147: "沖合で高い津波を観測したため津波警報を切り替えました。",
        148: "沖合で高い津波を観測したため予想される津波の高さを切り替えました。",
        149: "ただちに避難してください。",
        150: "南海トラフ地震臨時情報を発表しています。",
        201: "強い揺れに警戒してください。",
        211: "津波警報等(大津波警報・津波警報あるいは津波注意報)を発表中です。",
        212: "この地震により、日本の沿岸では若干の海面変動があるかもしれませんが、被害の心配はありません。",
        213: "今後もしばらく海面変動が続くと思われますので、海水浴や磯釣り等を行う際は注意してください。",
        214: "今後もしばらく海面変動が続くと思われますので、磯釣り等を行う際は注意してください。",
        215: "この地震による津波の心配はありません。",
        216: "震源が海底の場合、津波が発生するおそれがあります。",
        217: "今後の情報に注意してください。",
        221: "太平洋の広域に津波発生の可能性があります。",
        222: "太平洋で津波発生の可能性があります。",
        223: "北西太平洋で津波発生の可能性があります。",
        224: "インド洋の広域に津波発生の可能性があります。",
        225: "インド洋で津波発生の可能性があります。",
        226: "震源の近傍で津波発生の可能性があります。",
        227: "震源の近傍で小さな津波発生の可能性がありますが、被害をもたらす津波の心配はありません。",
        228: "一般的に、この規模の地震が海域の浅い領域で発生すると、津波が発生することがあります。",
        229: "日本への津波の有無については現在調査中です。",
        230: "この地震による日本への津波の影響はありません。",
        241: "この地震について、緊急地震速報を発表しています。",
        242: "この地震について、緊急地震速報を発表しています。この地震の最大震度は2でした。",
        243: "この地震について、緊急地震速報を発表しています。この地震の最大震度は1でした。",
        244: "この地震について、緊急地震速報を発表しています。この地震で震度1以上は観測されていません。",
        245: "この地震で緊急地震速報を発表しましたが、強い揺れは観測されませんでした。",
        256: "震源要素を訂正します。",
        500: "その他の防災上の留意事項",
        # "NNN*": "防災上の留意事項(コード番号：NNN)",
    },
    undefined="防災上の留意事項(コード番号：%d)"
)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcx_camf_a8_hazard_duration = QzssDcrDefinition(
    {
        0: 'Unknown',
        1: 'Duration < 6H',
        2: '6H <= Duration < 12H',
        3: '12H <= Duration < 24H',
    },
    undefined='Undefined Hazard Duration (Code: %d)'
)



from .qzss_dcr_definition import QzssDcrDefinition

qzss_dcr_jma_typhoon_scale_category = QzssDcrDefinition(
    {
        0: "なし",
        1: "大型",
        2: "超大型",
        15: "その他の大きさ階級分類",
        # "NN*": "大きさ階級分類(コード番号：NN)",
    },
    undefined="大きさ階級分類(コード番号：%d)"
)
